@@ -16,8 +16,9 @@ from mo_future import first, is_text, long, string_types, text
 from mo_parsing import listwrap
 
 from mo_sql_parsing.keywords import RESERVED, join_keywords, precedence
-from mo_sql_parsing.utils import binary_ops
+from mo_sql_parsing.utils import binary_ops, is_set_op
 
+MAX_PRECEDENCE = 100
 VALID = re.compile(r"^[a-zA-Z_]\w*$")
 
 
@@ -62,9 +63,9 @@ def escape(ident, ansi_quotes, should_quote):
     return ".".join(esc(f) for f in split_field(ident))
 
 
-def Operator(op):
-    prec = precedence[binary_ops[op]]
-    op = " {0} ".format(op).upper()
+def Operator(_op):
+    prec = precedence[binary_ops[_op]]
+    op = " {0} ".format(_op).upper()
 
     def func(self, json):
         acc = []
@@ -74,35 +75,48 @@ def Operator(op):
             k, v = first(json.items())
             json = [k, {"literal": v}]
 
-        for v in json if isinstance(json, list) else [json]:
-            sql = self.dispatch(v)
+        for v in listwrap(json):
+            sql = self.dispatch(v, prec)
             if isinstance(v, (text, int, float, long)):
                 acc.append(sql)
                 continue
-            # Error when formating NULL operators #7
             if v is None:
                 acc.append("NULL")
                 continue
-            p = precedence.get(first(v.keys()))
-            if p is None:
+            if isinstance(v, list):
                 acc.append(sql)
                 continue
-            if p >= prec:
-                acc.append("(" + sql + ")")
-            else:
-                acc.append(sql)
+            acc.append(isolate(v, sql, prec))
         return op.join(acc)
 
     return func
 
 
+def isolate(expr, sql, prec):
+    """
+    RETURN sql IN PARENTHESIS IF PREEDENCE > prec
+    :param expr: expression to isolate
+    :param sql: sql to return
+    :param prec: current precedence
+    """
+    if is_text(expr):
+        return sql
+    ps = [p for k in expr.keys() for p in [precedence.get(k)] if p is not None]
+    if not ps:
+        return sql
+    elif min(ps) >= prec:
+        return f"({sql})"
+    else:
+        return sql
+
+
 class Formatter:
 
     clauses = [
-        "with_",
+        "with",
         "select",
         "select_distinct",
-        "from_",
+        "from",
         "where",
         "groupby",
         "having",
@@ -111,7 +125,7 @@ class Formatter:
         "offset",
     ]
 
-    # simple operators
+    # infix operators
     _concat = Operator("||")
     _mul = Operator("*")
     _div = Operator("/")
@@ -124,42 +138,45 @@ class Formatter:
     _gte = Operator(">=")
     _lte = Operator("<=")
     _eq = Operator("=")
+    _in = Operator("in")
+    _nin = Operator("not in")
     _or = Operator("or")
     _and = Operator("and")
     _binary_and = Operator("&")
     _binary_or = Operator("|")
+    _like = Operator("like")
+    _not_like = Operator("not_like")
+    _rlike = Operator("rlike")
+    _not_rlike = Operator("not_rlike")
+    _union = Operator("union")
+    _union_all = Operator("union all")
+    _intersect = Operator("intersect")
+    _minus = Operator("minus")
+    _except = Operator("except")
 
     def __init__(self, ansi_quotes=True, should_quote=should_quote):
         self.ansi_quotes = ansi_quotes
         self.should_quote = should_quote
 
     def format(self, json):
-        if "union" in json:
-            return self.union(json["union"])
-        elif "union_all" in json:
-            return self.union_all(json["union_all"])
-        else:
-            return self.query(json)
+        return self.dispatch(json, 50)
 
-    def dispatch(self, json):
+    def dispatch(self, json, prec=100):
         if isinstance(json, list):
             return self.sql_list(json)
         if isinstance(json, dict):
             if len(json) == 0:
                 return ""
             elif "value" in json:
-                return self.value(json)
-            elif "from" in json:
-                # Nested queries
-                return "({})".format(self.format(json))
-            elif "select" in json:
-                # Nested queries
-                return "({})".format(self.format(json))
-            elif "select_distinct" in json:
-                # Nested queries
-                return "({})".format(self.format(json))
-            elif "on" in json:
+                return self.value(json, prec)
+            elif "join" in json:
                 return self._join_on(json)
+            elif json.keys() & set(["select", "select_distinct", "from"]):
+                sql = self.query(json)
+                if prec > precedence['from']:
+                    return sql
+                else:
+                    return f"({sql})"
             elif "null" in json:
                 return "NULL"
             else:
@@ -171,17 +188,20 @@ class Formatter:
 
         return text(json)
 
-    def sql_list(self, json):
-        return "(" + ", ".join(self.dispatch(element) for element in json) + ")"
+    def sql_list(self, json, prec=precedence['from']-1):
+        sql = ", ".join(self.dispatch(element, prec=MAX_PRECEDENCE) for element in json)
+        if prec >= precedence['from']:
+            return sql
+        else:
+            return f"({sql})"
 
-    def value(self, json):
-        parts = [self.dispatch(json["value"])]
+    def value(self, json, prec=precedence['from']):
+        parts = [self.dispatch(json["value"], prec)]
         if "name" in json:
             parts.extend(["AS", self.dispatch(json["name"])])
         return " ".join(parts)
 
     def op(self, json):
-
         if len(json) > 1:
             raise Exception("Operators should have only one key!")
         key, value = list(json.items())[0]
@@ -198,7 +218,7 @@ class Formatter:
         if isinstance(value, dict) and len(value) == 0:
             return (
                 key.upper() + "()"
-            )  # NOT SURE IF AN EMPTY dict SHOULD BE DELT WITH HERE, OR IN self.dispatch()
+            )  # NOT SURE IF AN EMPTY dict SHOULD BE DELT WITH HERE, OR IN self.format()
         else:
             params = ", ".join(self.dispatch(p) for p in listwrap(value))
             return f"{key.upper()}({params})"
@@ -207,47 +227,13 @@ class Formatter:
         return "~{0}".format(self.dispatch(value))
 
     def _exists(self, value):
-        return "{0} IS NOT NULL".format(self.dispatch(value))
+        return "{0} IS NOT NULL".format(self.dispatch(value, precedence['is']))
 
     def _missing(self, value):
-        return "{0} IS NULL".format(self.dispatch(value))
-
-    def _like(self, pair):
-        return "{0} LIKE {1}".format(self.dispatch(pair[0]), self.dispatch(pair[1]))
-
-    def _not_like(self, pair):
-        return "{0} NOT LIKE {1}".format(self.dispatch(pair[0]), self.dispatch(pair[1]))
-
-    def _rlike(self, pair):
-        return "{0} RLIKE {1}".format(self.dispatch(pair[0]), self.dispatch(pair[1]))
-
-    def _not_rlike(self, pair):
-        return "{0} NOT RLIKE {1}".format(
-            self.dispatch(pair[0]), self.dispatch(pair[1])
-        )
-
-    def _is(self, pair):
-        return "{0} IS {1}".format(self.dispatch(pair[0]), self.dispatch(pair[1]))
+        return "{0} IS NULL".format(self.dispatch(value, precedence['is']))
 
     def _collate(self, pair):
-        return "{0} COLLATE {1}".format(self.dispatch(pair[0]), pair[1])
-
-    def _in(self, json):
-        value, many = json
-
-        value = self.dispatch(value)
-        many = self.dispatch(many)
-
-        return "{0} IN {1}".format(value, many)
-
-    def _nin(self, json):
-        valid = self.dispatch(json[1])
-        # `(10, 11, 12)` does not get parsed as literal, so it's formatted as
-        # `10, 11, 12`. This fixes it.
-        if not valid.startswith("("):
-            valid = "({0})".format(valid)
-
-        return "{0} NOT IN {1}".format(json[0], valid)
+        return "{0} COLLATE {1}".format(self.dispatch(pair[0], precedence['collate']), pair[1])
 
     def _case(self, checks):
         parts = ["CASE"]
@@ -273,14 +259,17 @@ class Formatter:
 
     def _between(self, json):
         return "{0} BETWEEN {1} AND {2}".format(
-            self.dispatch(json[0]), self.dispatch(json[1]), self.dispatch(json[2])
+            self.dispatch(json[0], precedence['between']), self.dispatch(json[1], precedence['between']), self.dispatch(json[2], precedence['between'])
         )
 
     def _not_between(self, json):
         return "{0} NOT BETWEEN {1} AND {2}".format(
-            self.dispatch(json[0]), self.dispatch(json[1]), self.dispatch(json[2])
+            self.dispatch(json[0], precedence['between']), self.dispatch(json[1], precedence['between']), self.dispatch(json[2], precedence['between'])
         )
 
+    def _distinct(self, json):
+        return "DISTINCT " + ", ".join(self.dispatch(v) for v in listwrap(json))
+
     def _join_on(self, json):
         detected_join = join_keywords & set(json.keys())
         if len(detected_join) == 0:
@@ -305,16 +294,11 @@ class Formatter:
             acc.append(self.dispatch(json["using"]))
         return " ".join(acc)
 
-    def union(self, json):
-        return " UNION ".join(self.query(query) for query in json)
-
-    def union_all(self, json):
-        return " UNION ALL ".join(self.query(query) for query in json)
-
     def query(self, json):
         return " ".join(
             part
             for clause in self.clauses
+            if clause in json
             for part in [getattr(self, clause)(json)]
             if part
         )
@@ -325,70 +309,60 @@ class Formatter:
             if not isinstance(with_, list):
                 with_ = [with_]
             parts = ", ".join(
-                "{0} AS {1}".format(part["name"], self.dispatch(part["value"]))
+                "{0} AS ({1})".format(part["name"], self.dispatch(part["value"]))
                 for part in with_
             )
             return "WITH {0}".format(parts)
 
     def select(self, json):
-        if "select" in json:
-            param = ", ".join(self.dispatch(s) for s in listwrap(json["select"]))
-            if "top" in json:
-                top = self.dispatch(json["top"])
-                return f"SELECT TOP ({top}) {param}"
-            else:
-                return f"SELECT {param}"
-
-    def select_distinct(self, json):
-        if "select_distinct" in json:
-            return "SELECT DISTINCT {0}".format(self.dispatch(json["select_distinct"]))
+        param = ", ".join(self.dispatch(s) for s in listwrap(json["select"]))
+        if "top" in json:
+            top = self.dispatch(json["top"])
+            return f"SELECT TOP ({top}) {param}"
+        else:
+            return f"SELECT {param}"
 
     def from_(self, json):
         is_join = False
-        if "from" in json:
-            from_ = json["from"]
-            if "union" in from_:
-                return self.union(from_["union"])
-            if not isinstance(from_, list):
-                from_ = [from_]
+        from_ = json["from"]
+        if isinstance(from_, dict) and is_set_op & from_.keys():
+            return self.op(from_)
 
-            parts = []
-            for token in from_:
-                if join_keywords & set(token):
-                    is_join = True
-                    parts.append(self._join_on(token))
-                else:
-                    parts.append(self.dispatch(token))
-            joiner = " " if is_join else ", "
-            rest = joiner.join(parts)
-            return "FROM {0}".format(rest)
+        from_ = listwrap(from_)
+        parts = []
+        for token in from_:
+            if join_keywords & set(token):
+                is_join = True
+                parts.append(self._join_on(token))
+            else:
+                parts.append(self.dispatch(token, precedence["from"]))
+        joiner = " " if is_join else ", "
+        rest = joiner.join(parts)
+        return "FROM {0}".format(rest)
 
     def where(self, json):
-        if "where" in json:
-            return "WHERE {0}".format(self.dispatch(json["where"]))
+        return "WHERE {0}".format(self.dispatch(json["where"]))
 
     def groupby(self, json):
-        if "groupby" in json:
-            param = ", ".join(self.dispatch(s) for s in listwrap(json["groupby"]))
-            return f"GROUP BY {param}"
+        param = ", ".join(self.dispatch(s) for s in listwrap(json["groupby"]))
+        return f"GROUP BY {param}"
 
     def having(self, json):
-        if "having" in json:
-            return "HAVING {0}".format(self.dispatch(json["having"]))
+        return "HAVING {0}".format(self.dispatch(json["having"]))
 
     def orderby(self, json):
-        if "orderby" in json:
-            param = ", ".join(
-                (self.dispatch(s["value"]) + " " + s.get("sort", "").upper()).strip()
-                for s in listwrap(json["orderby"])
-            )
-            return f"ORDER BY {param}"
+        param = ", ".join(
+            (self.dispatch(s["value"]) + " " + s.get("sort", "").upper()).strip()
+            for s in listwrap(json["orderby"])
+        )
+        return f"ORDER BY {param}"
 
     def limit(self, json):
-        if "limit" in json:
-            if json["limit"]:
-                return "LIMIT {0}".format(self.dispatch(json["limit"]))
+        return "LIMIT {0}".format(self.dispatch(json["limit"]))
 
     def offset(self, json):
-        if "offset" in json:
-            return "OFFSET {0}".format(self.dispatch(json["offset"]))
+        return "OFFSET {0}".format(self.dispatch(json["offset"]))
+
+
+setattr(Formatter, "with", Formatter.with_)
+setattr(Formatter, "from", Formatter.from_)
