@@ -182,8 +182,8 @@ def to_tuple_call(tokens):
     if all(isinstance(r, number_types) for r in tokens):
         return [tokens]
     if all(
-        isinstance(r, number_types) or (is_data(r) and "literal" in r.keys())
-        for r in tokens
+            isinstance(r, number_types) or (is_data(r) and "literal" in r.keys())
+            for r in tokens
     ):
         candidate = {"literal": [r["literal"] if is_data(r) else r for r in tokens]}
         return candidate
@@ -212,16 +212,27 @@ binary_ops = {
     "!=": "neq",
     "<>": "neq",
     "not in": "nin",
+    "in": "in",
     "is not": "neq",
     "is": "eq",
-    "similar to": "similar_to",
-    "not like": "not_like",
-    "not rlike": "not_rlike",
-    "not simlilar to": "not_similar_to",
+    "similar_to": "similar_to",
+    "like": "like",
+    "rlike": "rlike",
+    "not_like": "not_like",
+    "not_rlike": "not_rlike",
+    "not_simlilar_to": "not_similar_to",
     "or": "or",
     "and": "and",
+    "union": "union",
+    "union_all": "union_all",
+    "union all": "union_all",
+    "except": "except",
+    "minus": "minus",
+    "intersect": "intersect",
 }
 
+is_set_op = ("union", "union_all", "except", "minus", "intersect")
+
 
 def to_trim_call(tokens):
     frum = tokens["from"]
@@ -246,6 +257,7 @@ def to_json_call(tokens):
         tokens.start,
         tokens.end,
         [Call(op, params, {"ignore_nulls": ignore_nulls})],
+        tokens.failures
     )
 
 
@@ -307,7 +319,7 @@ def to_expression_call(tokens):
         return
 
     expr = ParseResults(
-        tokens.type, tokens.start, tokens.end, listwrap(tokens["value"])
+        tokens.type, tokens.start, tokens.end, listwrap(tokens["value"]), tokens.failures
     )
     return expr
 
@@ -344,7 +356,8 @@ def to_select_call(tokens):
         return ["*"]
 
     if value["over"] or value["within"]:
-        output = ParseResults(tokens.type, tokens.start, tokens.end, value.tokens)
+        output = ParseResults(tokens.type, tokens.start, tokens.end, value.tokens, tokens.failures
+                              )
         output["name"] = tokens["name"]
         return output
     else:
@@ -410,8 +423,8 @@ def to_string(tokens):
 # NUMBERS
 realNum = (
     Regex(r"[+-]?(\d+\.\d*|\.\d+)([eE][+-]?\d+)?")
-    .set_parser_name("float")
-    .addParseAction(lambda t: float(t[0]))
+        .set_parser_name("float")
+        .addParseAction(lambda t: float(t[0]))
 )
 
 
@@ -427,8 +440,8 @@ intNum = (
 )
 hexNum = (
     Regex(r"0x[0-9a-fA-F]+")
-    .set_parser_name("hex")
-    .addParseAction(lambda t: {"hex": t[0][2:]})
+        .set_parser_name("hex")
+        .addParseAction(lambda t: {"hex": t[0][2:]})
 )
 
 # STRINGS
